from typing import TypedDict, Optional

from davidkhala.ai.api import API


class Leaderboard(TypedDict):
    url:Optional[str]
    name:Optional[str]

class OpenRouter(API):
    @property
    def free_models(self) -> list[str]:
        return list(
            map(lambda model: model['id'],
                filter(lambda model: model['id'].endswith(':free'), self.list_models())
                )
        )

    def __init__(self, api_key: str, models: list[str] = None, *,
                 leaderboard: Leaderboard = None):

        super().__init__(api_key, 'https://openrouter.ai/api')
        self.leaderboard = leaderboard
        if models is None:
            models = [self.free_models[0]]
        self.models = models

    def pre_request(self, headers: dict, data: dict):
        if self.leaderboard is not None:
            headers["HTTP-Referer"] = self.leaderboard['url'],  # Optional. Site URL for rankings on openrouter.ai.
            headers["X-Title"] = self.leaderboard['name'],  # Optional. Site title for rankings on openrouter.ai.
        if len(self.models) > 1:
            data["models"] = self.models
        else:
            data["model"] = self.models[0]



import datetime
from abc import abstractmethod, ABC

import requests


class API(ABC):
    def __init__(self, api_key: str, base_url: str):
        self.base_url = base_url+'/v1'
        self.model = None
        self.headers = {
            "Authorization": f"Bearer {api_key}",
        }
    @property
    @abstractmethod
    def free_models(self)->list[str]:
        ...

    @abstractmethod
    def pre_request(self, headers: dict, data: dict):
        data["model"] = self.model
    def chat(self, prompt, system_prompt: str = None):


        messages = [
            {
                "role": "user",
                "content": prompt
            }
        ]
        if system_prompt is not None:
            messages.append({
                "role": "system",
                "content": system_prompt
            })
        json = {
            "messages": messages
        }
        self.pre_request(self.headers, json)
        # timeout=50 to cater siliconflow
        response = requests.post(f"{self.base_url}/chat/completions", headers=self.headers, json=json, timeout=50)
        parsed_response = API.parse(response)


        return {
            "data": list(map(lambda x: x['message']['content'], parsed_response['choices'])),
            "meta": {
                "usage": parsed_response['usage'],
                "created": datetime.datetime.fromtimestamp(parsed_response['created'])
            }
        }
    @staticmethod
    def parse(response):
        parsed_response = response.json()

        match parsed_response:
            case dict():
                err = parsed_response.get('error')
                if err is not None:
                    raise Exception(err)
            case str():
                raise Exception(parsed_response)
        return parsed_response
    def list_models(self):
        response = requests.get(f"{self.base_url}/models", headers=self.headers)
        return API.parse(response)['data']


import runpy
from abc import ABC
from typing import Union, Optional, Literal, List

from openai import OpenAI, AsyncOpenAI


class Client(ABC):
    api_key: str
    base_url: str
    model: Optional[str]
    messages = []
    client: OpenAI

    def as_chat(self, model, sys_prompt: str = None):
        self.model = model
        if sys_prompt is not None:
            self.messages = [{"role": "system", "content": sys_prompt}]

    def as_embeddings(self, model, encoding_format: str = "float"):
        self.model = model

    def connect(self):
        self.client.models.list()

    def encode(self, _input: str, _format: Literal["float", "base64"] = "float")-> List[float]:
        response = self.client.embeddings.create(
            model=self.model,
            input=_input,
            encoding_format=_format
        )
        return response.data[0].embedding

    def chat(self, user_prompt, image: str = None):

        message = {
            "role": "user"
        }
        if image is None:
            message['content'] = user_prompt
        else:
            message['content'] = [
                {"type": "text", "text": user_prompt},
                {
                    "type": "image_url",
                    "image_url": {
                        "url": image,
                    }
                },
            ]
        response = self.client.chat.completions.create(
            model=self.model,
            messages=[
                *self.messages,
                message
            ],
        )
        return response.choices[0].message.content

    def disconnect(self):
        self.client.close()


def with_opik(instance: Union[OpenAI, AsyncOpenAI]):
    from opik.integrations.openai import track_openai
    runpy.run_path('../opik.py')
    return track_openai(instance)
